/*
 * Revision History:
     27/10/12         start
 * Description:
     Usart1 using interupts for rx and tx with buffering
     re-transmits rx chr back on tx
 * Test configuration:
     Device:          STM32F107VC
     dev.board:       easymx v7 for stm32 arm
     Oscillator:      HSE-PLL, 72.000MHz
     Ext. Modules:    None.
     SW:              mikroC PRO for ARM v2.5
 * NOTES:
     - Turn ON USB UART1 switches (SW12.1 and SW12.2)
     - uart setup for for 9600:8N1
 */

/*******************************************************************************/
// GLOBALS
/*******************************************************************************/
#define RXD_BUFF    20
char RxData[RXD_BUFF] ;
unsigned short RxChr ;
unsigned int RxWp, RxRp, RxCnt ;

#define TXD_BUFF     20
char TxData[TXD_BUFF] ;
unsigned int TxWp, TxRp, TxCnt ;

/*******************************************************************************/
// FUNCTION PROTOTYPES
/*******************************************************************************/

unsigned short TxChr(unsigned short chr) ;

/*******************************************************************************/
// INTERUPTS
/*******************************************************************************/
void Uart1_interrupt() iv IVT_INT_USART1 ics ICS_AUTO {
GPIOE_ODR.B8 = 1 ;            // for debug & timing

if(RXNEIE_USART1_CR1_bit == 1) {    // are Rx data ready ints enabled ?
 if(RXNE_USART1_SR_bit == 1) {      // yes, is interupt pending ?
  RXNE_USART1_SR_bit = 0 ;          // yes, reset int flag
  RxChr = USART1_DR ;               // save rx chr
  if(RxCnt < RXD_BUFF) {            // any room in rx buffer ?
   RxData[RxWp] = RxChr ;           // yes,put copy in buffer
   RxWp++ ;                         // inc pntr
   if(RxWp >= RXD_BUFF)             // past end of buffer ?
    RxWp = 0 ;                      // yes, wrap to start
   RxCnt++ ;                        // inc data cnt
   }
  }
 }

if(TXEIE_USART1_CR1_bit == 1) {     // are TX buffer empty ints enabled ?
 if(TXE_USART1_SR_bit == 1) {       // yes, is interupt pending ?
  TXE_USART1_SR_bit = 0 ;           // yes, reset int flag
  if(TxCnt > 0) {                   // any data waiting ?
   USART1_DR = TxData[TxRp] ;       // yes, tx next data
   TxRp++ ;                         // inc rd pntr
   if(TxRp >= TXD_BUFF)             // past end of buffer ?
    TxRp = 0 ;                      // yes, wrap to start
   TxCnt-- ;                        // dec data cnt
   if(TxCnt == 0)                   // last data in buffer ?
    TXEIE_USART1_CR1_bit = 0 ;      // yes, disable uart TX interupts
   }
  }
 }

GPIOE_ODR = 0;                 // for debug & timing
}

/*******************************************************************************/
// MAIN
/*******************************************************************************/
void main() {

GPIO_Digital_Output(&GPIOE_BASE, _GPIO_PINMASK_ALL);  // Enable digital output on PORTE for debugging
GPIOE_ODR = 0;

GPIO_Alternate_Function_Enable(&_GPIO_MODULE_USART1_PA9_10) ;  // Enable UART1 on GPIO_PORTA

RxWp = RxRp = RxCnt = 0 ;         // init buffer pointers
TxWp = TxRp = TxCnt = 0 ;

RCC_APB2ENR.USART1EN = 1 ;        // Turn on USART1 module clocks
USART1_CR1.UE = 1 ;               // Enable USART1
USART1_CR1.M = 0 ;                // set 8 bit data len  (reset default, can be ignored)
USART1_CR1.PCE = 0 ;              // set no parity       (reset default, can be ignored)
USART1_CR2.STOP0 = 0 ;            // set 1 stop bit      (reset default, can be ignored)
USART1_CR2.STOP1 = 0 ;            //    ditto            (reset default, can be ignored)
USART1_BRR = 7500 ;               // set baud rate 9600 (osc freq / baudrate)
USART1_CR1.RE = 1 ;               // enable rx
USART1_CR1.TE = 1 ;               // enable tx

NVIC_IntEnable(IVT_INT_USART1);   // Enable UART1 interrupts
USART1_CR1.TXEIE = 0 ;            // disable tx empty interupts     (reset default, can be ignored)
RXNE_USART1_SR_bit = 0 ;          // clear any pending rx data flag
RXNEIE_USART1_CR1_bit = 1 ;       // enable rx data ready interupts

while(1) {

 if(RxCnt > 0) {                  // any rx data ready ?
  TxChr(RxData[RxRp]) ;           // yes, send next chr
  RxRp++ ;                        // inc rd pntr
  if(RxRp >= RXD_BUFF)            // past end of buffer ?
   RxRp = 0 ;                     // yes, wrap to start
  RXNEIE_USART1_CR1_bit = 0 ;     // stop rx data ready interupts
  RxCnt-- ;                       // dec data cnt
  RXNEIE_USART1_CR1_bit = 1 ;     // restart rx ints
  }
  
 }  // end while
 
} // end main

/*******************************************************************************/
unsigned short TxChr(unsigned short chr)
// return true if ok, return false if no space in tx buffer
{
if(TxCnt < TXD_BUFF) {           // any space left in buffer ?
 TxData[TxWp] = chr ;            // yes, save data
 TxWp++ ;                        // inc write pntr
 if(TxWp >= TXD_BUFF)            // past end of buffer ?
  TxWp = 0 ;                     // yes, wrap to start
 TXEIE_USART1_CR1_bit = 0 ;      // stop Tx buffer empty ints
 TxCnt++ ;                       // inc data cnt
 TXEIE_USART1_CR1_bit = 1 ;      // yes, re-start ints
 return(1) ;
 }
 
else
 return(0) ;                     // no space in buffer - return false
}

/*******************************************************************************/